@@ -45,6 +45,8 @@ function Debugger(sel) {
     this.breakpointConditionTemplate = Handlebars.compile( $('#breakpoint-condition-template').html() );
     this.breakpointActionTemplate = Handlebars.compile( $('#breakpoint-action-template').html() );
     this.breakpointListItemTemplate = Handlebars.compile( $('#breakpoint-list-item-template').html() );
+    this.forkedChildModalTemplate = Handlebars.compile($('#forked-child-modal-template').html() );
+    this.saveLoadBreakpointsModalTemplate = Handlebars.compile($('#save-load-breakpoints-modal-template').html() );
 
     // The step in, over, run buttons
     this.controlButtons = $('.control-button');
@@ -54,7 +56,8 @@ function Debugger(sel) {
                             .addHandler('termination', this.programWasTerminated.bind(this))
                             .addHandler('evalresult', this.watchExpressionResult.bind(this))
                             .addHandler('hangup', this.hangup.bind(this))
-                            .addHandler('getvar', this.drawPerlVarPopover.bind(this));
+                            .addHandler('getvar', this.drawPerlVarPopover.bind(this))
+                            .addHandler('child_process', this.childProcessForked.bind(this));
 
 
         this.getProgramName();
@@ -86,6 +89,8 @@ function Debugger(sel) {
                     this.scrollToBreakpoint.bind(this));
         this.elt.on('hover', '.popup-perl-var',
                     this.popoverPerlVar.bind(this));
+        this.elt.on('click', '.save-load-breakpoints',
+                    this.saveLoadBreakpoints.bind(this));
 
         $('#breakpoint-container-handle').click(function(e) {
             var breakpointPane = $('#breakpoint-container'),
@@ -203,6 +208,43 @@ function Debugger(sel) {
         this.programTerminated = true;
     };
 
+    this._defaultBreakpointFileName = function () {
+        return this.programName + '.hdb';
+    };
+    this.saveLoadBreakpoints = function(e) {
+        e.preventDefault();
+        var isSave = $(e.currentTarget).attr('id') === 'save-breakpoints';
+        var modal = $( this.saveLoadBreakpointsModalTemplate({
+                                    action: isSave ? 'Save' : 'Load',
+                                    filename: this._defaultBreakpointFileName()
+                        }))
+                        .appendTo(this.elt)
+                        .modal({ backdrop: true, keyboard: true, show: true })
+                        .focus();
+
+        modal.on('hidden', function(e) {
+            modal.remove();
+        });
+
+        var form = modal.find('form'),
+            messageHandler = this.messageHandler;
+        form.submit(function(e) {
+            e.preventDefault();
+            var savefile = form.find('[name="filename"]').val();
+            messageHandler.ajaxSession(
+                   {url: isSave ? 'saveconfig' : 'loadconfig',
+                    type: 'POST',
+                    dataType: 'json',
+                    data: { f: savefile },
+                },
+                function(data) {
+                    modal.modal('hide');
+                    messageHandler.sessionDone();
+                }
+            );
+        });
+    };
+
     // Toggle the breakpoint on a line
     this.breakableLineClicked = function(e) {
         var $elt = $(e.target),
@@ -547,6 +589,7 @@ function Debugger(sel) {
 
     this.programNameUpdated = function(name) {
         this.elt.find('li#program-name a').html(name);
+        this.programName = name;
     };
     this.getProgramName = function() {
         $.ajax({url: 'program_name',
@@ -633,7 +676,7 @@ function Debugger(sel) {
 
         var n;
         for (n = 0; n < frame.args.length; n++) {
-            subArgs.push(('render' in frame.args[n]) ? frame.args[n].render() : frame.args[n] );
+            subArgs.push(('render' in frame.args[n]) ? frame.args[n].render('condensed') : frame.args[n] );
         }
 
         if (isSameFrame) {
@@ -795,6 +838,36 @@ function Debugger(sel) {
                 +error+'": '+xhdr.responseText);
     };
 
+    this.childProcessForked = function(data) {
+        var pid = data.pid,
+            uri = data.uri,
+            run = data.run;
+
+        //alert('Child process '+pid+' forked.  <a href="'+uri+'">Click here</a> to debug it');
+        var modal = $( this.forkedChildModalTemplate({pid: pid, uri: uri, run: run}))
+                        .appendTo(this.elt)
+                        .modal({ backdrop: true, keyboard: true, show: true })
+                        .focus();
+        modal.on('click', '.btn', function(e) {
+            var $elt = $(e.target);
+            modal.on('hidden', function(e) {
+                modal.remove();
+            });
+            modal.modal('hide');
+            if ($elt.hasClass('run-child')) {
+                // telling the child process to run without stopping.  We'll send
+                // the request by hand, here
+                $.ajax({url: $elt.attr('href'),
+                        method: 'GET',
+                        error: this.ajaxError
+                    });
+                e.preventDefault();
+            }
+            // for the "open" button, go ahead and let the browser follow the link
+            // it will open in a new window/tab
+        });
+    };
+
     this.initialize();
 
     function BreakpointManager() {
@@ -907,6 +980,10 @@ function Debugger(sel) {
 // Routes incoming messages to the right place
 function MessageHandler() {
     this.handlers = {};
+    this.requestors = {};
+
+    this.req_id = 1;
+    this.current_req_id = undefined;
 
     this.addHandler = function(type, cb) {
         this.handlers[type] = cb;
@@ -914,16 +991,46 @@ function MessageHandler() {
     };
 
     this.handle = function(data) {
-        var that=this;
+        var h = this;
+        function runMessageCallback(message) {
+            var data = message.data,
+                req_id = message.rid,
+                type = message.type;
+
+            if ((req_id !== null) && (req_id !== undefined)) {
+                h.current_req_id = req_id;
+                h.requestors[ req_id ](data);
+                h.current_req_id = undefined;
+            } else {
+                h.handlers[ type ](data);
+            }
+        };
+
+        var i;
         if ('forEach' in data) {
-            data.forEach(function(message) {
-                that.handlers[ message.type ]( message.data );
-            });
+            for( i = 0; i < data.length; i++) {
+                runMessageCallback(data[i]);
+            }
         } else {
-            that.handlers[ data.type ]( data.data );
+            runMessageCallback(data);
         }
     };
 
     this.consumer = this.handle.bind(this);
+
+    this.ajaxSession = function(params, cb) {
+        var req_id = this.req_id++;
+        this.requestors[req_id] = cb;
+
+        params.data['rid'] = req_id;
+        params.success = this.handle.bind(this);
+        return $.ajax(params);
+    };
+
+    this.sessionDone = function() {
+        if (this.current_req_id !== undefined) {
+            delete this.requestors[ this.current_req_id ];
+        }
+    };
 }
 
