@@ -1,7 +1,7 @@
 // Represents a Perl value
 function PerlValue() { }
-PerlValue.prototype.render = function() {
-    return this.renderHeader() + this.renderValue();
+PerlValue.prototype.render = function(view) {
+    return this.renderHeader(view) + this.renderValue(view);
 }
 
 PerlValue.parseFromEval = function(value) {
@@ -24,7 +24,7 @@ PerlValue.exception = function(value) {
     this.type = 'exception';
 };
 PerlValue.exception.prototype = new PerlValue();
-PerlValue.exception.prototype.renderValue = function() {
+PerlValue.exception.prototype.renderValue = function(view) {
     return '<span class="label label-important PerlValue">'+this.value+'</span>';
 };
 
@@ -34,7 +34,7 @@ PerlValue.scalar = function(value) {
     this.type = 'scalar';
 }
 PerlValue.scalar.prototype = new PerlValue();
-PerlValue.scalar.prototype.renderValue = function() {
+PerlValue.scalar.prototype.renderValue = function(view) {
     if ((this.value === undefined) || (this.value === null)) {
         return '<span class="label PerlValue"><i>undef</i></span>';
     } else {
@@ -50,11 +50,11 @@ PerlValue.SCALAR = function(value) {
     this.blessed = value.__blessed;
 }
 PerlValue.SCALAR.prototype = new PerlValue();
-PerlValue.SCALAR.prototype.renderHeader = function() {
+PerlValue.SCALAR.prototype.renderHeader = function(view) {
     return '<span>' + this.refHeaderString() + '</span>';
 };
-PerlValue.SCALAR.prototype.renderValue = function() {
-    return '<span class="PerlValue">' + this.value.render() + '</span>';
+PerlValue.SCALAR.prototype.renderValue = function(view) {
+    return '<span class="PerlValue">' + this.value.render(view) + '</span>';
 }
 
 // ARRAY ref values
@@ -70,16 +70,21 @@ PerlValue.ARRAY = function(value) {
     }
 }
 PerlValue.ARRAY.prototype = new PerlValue();
-PerlValue.ARRAY.prototype.renderHeader = function() {
+PerlValue.ARRAY.prototype.renderHeader = function(view) {
     return '<span>' + this.refHeaderString() + '</span>'
-            + ' <span class="badge badge-info expr-collapse-button">'
-            + this.count +'</span>';
+            + ' <span class="badge badge-info'
+            + (view === 'condensed' ? '' : ' expr-collapse-button')
+            + '">' + this.count +'</span>';
 }
-PerlValue.ARRAY.prototype.renderValue = function() {
+PerlValue.ARRAY.prototype.renderValue = function(view) {
+    if (view === 'condensed') {
+        return '';
+    }
+
     var html = '<dl class="PerlValue">';
     for(var i = 0; i < this.value.length; i++) {
         html += '<dt>' + i + '</dt><dd>'
-                + this.value[i].render() + '</dd>';
+                + this.value[i].render(view) + '</dd>';
     }
     return html + '</dl>';
 }
@@ -101,16 +106,21 @@ PerlValue.HASH = function(value) {
     }
 }
 PerlValue.HASH.prototype = new PerlValue();
-PerlValue.HASH.prototype.renderHeader = function() {
+PerlValue.HASH.prototype.renderHeader = function(view) {
     return '<span>' + this.refHeaderString() + '</span>'
-            + ' <span class="badge badge-info expr-collapse-button">'
-            + this.count + '</span>';
+            + ' <span class="badge badge-info'
+            + (view === 'condensed' ? '' : ' expr-collapse-button')
+            + '">' + this.count + '</span>';
 }
-PerlValue.HASH.prototype.renderValue = function() {
+PerlValue.HASH.prototype.renderValue = function(view) {
+    if (view === 'condensed') {
+        return '';
+    }
+
     var html = '<dl class="PerlValue">',
         k;
     for (key in this.value) {
-        html += '<dt>' + key + '</dt><dd>' + this.value[key].render() + '</dd>';
+        html += '<dt>' + key + '</dt><dd>' + this.value[key].render(view) + '</dd>';
     }
     return html + '</dl>';
 }
@@ -130,16 +140,22 @@ PerlValue.GLOB = function(value) {
     }
 }
 PerlValue.GLOB.prototype = new PerlValue();
-PerlValue.GLOB.prototype.renderHeader = function() {
+PerlValue.GLOB.prototype.renderHeader = function(view) {
     return '<span>' + this.refHeaderString() + '</span>'
-            + ' <span class="badge badge-info expr-collapse-button">&plusmn;</span>';
+            + ' <span class="badge badge-info'
+            + (view === 'condensed' ? '">' : ' expr-collapse-button">&plusmn;')
+            + '</span>';
 }
-PerlValue.GLOB.prototype.renderValue = function() {
+PerlValue.GLOB.prototype.renderValue = function(view) {
+    if (view === 'condensed') {
+        return '';
+    }
+
     var html = '<dl class="PerlValue">',
         k;
     for (k in this.value) {
         html += '<dt><span class="label label-success">' + k + '</span></dt><dd>'
-                + this.value[k].render() + '</dd>';
+                + this.value[k].render(view) + '</dd>';
     }
     return html + '</dl>';
 }
@@ -152,7 +168,7 @@ PerlValue.CODE = function(value) {
     this.blessed = value.__blessed;
 }
 PerlValue.CODE.prototype = new PerlValue();
-PerlValue.CODE.prototype.render = function() {
+PerlValue.CODE.prototype.render = function(view) {
     return '<span>' + this.refHeaderString() + '</span>';
 }
 
@@ -164,12 +180,12 @@ PerlValue.REF = function(value) {
     this.blessed = value.__blessed;
 }
 PerlValue.REF.prototype = new PerlValue();
-PerlValue.REF.prototype.renderHeader = function() {
+PerlValue.REF.prototype.renderHeader = function(view) {
     return '<span>' + this.refHeaderString() + '</span>'
             + '<span class="badge badge-info expr-collapse-button">&plusmn;</span>';
 }
-PerlValue.REF.prototype.renderValue = function() {
-    return '<ul class="PerlValue"><li>' + this.value.render() + '</li></ul>';
+PerlValue.REF.prototype.renderValue = function(view) {
+    return '<ul class="PerlValue"><li>' + this.value.render(view) + '</li></ul>';
 }
 
 //Regexp (from qr() ) values
@@ -179,7 +195,7 @@ PerlValue.REGEXP = function(value) {
     this.type = value.__reftype;
     this.blessed = value.__blessed;
 }
-PerlValue.REGEXP.prototype.renderValue = function() {
+PerlValue.REGEXP.prototype.renderValue = function(view) {
     return '<span class="PerlValue">' + this.value + '</span>';
 }
 
@@ -194,5 +210,5 @@ PerlValue.prototype.refHeaderString = function() {
     html += this.type + '(0x' + this.refaddr.toString(16) + ')';
     return html;
 }
-PerlValue.prototype.renderHeader = function() { return '' };
-PerlValue.prototype.renderValue = function() { return '' };
+PerlValue.prototype.renderHeader = function(view) { return '' };
+PerlValue.prototype.renderValue = function(view) { return '' };
